@@ -1,13 +1,19 @@
 from .weighted_value_resolver import WeightedValueResolver
 from .config_value_wrapper import ConfigValueWrapper
 from .context import Context
+from .encryption import Encryption, DecryptionException
 import prefab_pb2 as Prefab
 import yaml
 import os
-
+import hashlib
+import isodate
+from ._internal_logging import InternalLogger
 
 VTV = Prefab.Config.ValueType.Value
 VTN = Prefab.Config.ValueType.Name
+CONFIDENTIAL_PREFIX = "*****"
+
+logger = InternalLogger(__name__)
 
 
 class EnvVarParseException(Exception):
@@ -20,6 +26,16 @@ class EnvVarParseException(Exception):
         )
 
 
+class MissingEnvVarException(Exception):
+    "Raised when an environment variable specified in a `provided` config value does not exist"
+
+    def __init__(self, config, env_var_name):
+        super().__init__(
+            "Environment variable %s referenced in config %s does not exist"
+            % (config.key, env_var_name)
+        )
+
+
 class UnknownConfigValueTypeException(Exception):
     "Raised when a config value of an unknown type is passed to the unwrapper"
 
@@ -37,10 +53,32 @@ class UnknownProvidedSourceException(Exception):
 class ConfigValueUnwrapper:
     def __init__(self, value, resolver, weighted_value_index=None):
         self.value = value
-        self.resolve = resolver
+        self.resolver = resolver
         self.weighted_value_index = weighted_value_index
 
-    def deepest_value(config_value, config, resolver, context=Context.get_current()):
+    def reportable_wrapped_value(self):
+        if self.value.confidential or len(self.value.decrypt_with) > 0:
+            # Unique hash for differentiation
+            hash = hashlib.md5()
+            hash.update(str(self.unwrap()).encode())
+            return ConfigValueUnwrapper(
+                ConfigValueWrapper.wrap(f"{CONFIDENTIAL_PREFIX}{hash.hexdigest()[:5]}"),
+                self.resolver,
+                self.weighted_value_index,
+            )
+        else:
+            return self
+
+    def reportable_value(self):
+        return self.reportable_wrapped_value().unwrap()
+
+    @staticmethod
+    def deepest_value(
+        config_value: Prefab.ConfigValue,
+        config,
+        resolver,
+        context=Context.get_current(),
+    ):
         if config_value and config_value.WhichOneof("type") == "weighted_values":
             value, index = WeightedValueResolver(
                 config_value.weighted_values.weighted_values,
@@ -58,11 +96,12 @@ class ConfigValueUnwrapper:
             if config_value.provided.source == Prefab.ProvidedSource.Value("ENV_VAR"):
                 raw = os.getenv(config_value.provided.lookup)
                 if raw is None:
-                    resolver.base_client.logger.log_internal(
-                        "warn",
-                        f"ENV Variable {config_value.provided.lookup} not found. Using empty string.",
-                    )
-                    return ConfigValueUnwrapper(ConfigValueWrapper.wrap(""), resolver)
+                    raise MissingEnvVarException(config, config_value.provided.lookup)
+                    # resolver.base_client.logger.log_internal(
+                    #     "warn",
+                    #     f"ENV Variable {config_value.provided.lookup} not found. Using empty string.",
+                    # )
+                    # return ConfigValueUnwrapper(ConfigValueWrapper.wrap(""), resolver)
                 else:
                     coerced = ConfigValueUnwrapper.coerce_into_type(
                         raw, config, config_value.provided.lookup
@@ -83,12 +122,35 @@ class ConfigValueUnwrapper:
         type = self.value.WhichOneof("type")
 
         if type in ["int", "string", "double", "bool", "log_level"]:
-            return getattr(self.value, type)
+            raw = getattr(self.value, type)
         elif type == "string_list":
-            return self.value.string_list.values
+            raw = self.value.string_list.values
+        elif type == "duration":
+            raw = isodate.parse_duration(self.value.duration.definition)
         else:
             raise UnknownConfigValueTypeException(type)
 
+        if self.value.decrypt_with != "":
+            decryption_key_evaluation = self.resolver.get(self.value.decrypt_with)
+            if (
+                decryption_key_evaluation is None
+                or decryption_key_evaluation.raw_config_value is None
+            ):
+                logger.warning(
+                    f"No value for decryption key {self.value.decrypt_with} found.",
+                )
+                return ""
+            else:
+                try:
+                    return Encryption(
+                        decryption_key_evaluation.unwrapped_value()
+                    ).decrypt(raw)
+                except Exception:
+                    raise DecryptionException("unable to decrypt value")
+        else:
+            return raw
+
+    @staticmethod
     def coerce_into_type(value_string, config, env_var_name):
         try:
             value_type = config.value_type
